@@ -1,7 +1,8 @@
-from Cython.Compiler.Visitor import VisitorTransform, temp_name_handle, CythonTransform
+from Cython.Compiler.Visitor import VisitorTransform, CythonTransform
 from Cython.Compiler.ModuleNode import ModuleNode
 from Cython.Compiler.Nodes import *
 from Cython.Compiler.ExprNodes import *
+from Cython.Compiler.UtilNodes import *
 from Cython.Compiler.TreeFragment import TreeFragment
 from Cython.Compiler.StringEncoding import EncodedString
 from Cython.Compiler.Errors import CompileError
@@ -80,7 +81,7 @@ class NormalizeTree(CythonTransform):
 class PostParseError(CompileError): pass
 
 # error strings checked by unit tests, so define them
-ERR_CDEF_INCLASS = 'Cannot assign default value to cdef class attributes'
+ERR_CDEF_INCLASS = 'Cannot assign default value to fields in cdef classes, structs or unions'
 ERR_BUF_LOCALONLY = 'Buffer types only allowed as function local variables'
 ERR_BUF_DEFAULTS = 'Invalid buffer defaults specification (see docs)'
 ERR_INVALID_SPECIALATTR_TYPE = 'Special attributes must not have a type declared'
@@ -130,31 +131,33 @@ class PostParse(CythonTransform):
 
     def visit_ModuleNode(self, node):
         self.scope_type = 'module'
+        self.scope_node = node
         self.visitchildren(node)
         return node
+
+    def visit_scope(self, node, scope_type):
+        prev = self.scope_type, self.scope_node
+        self.scope_type = scope_type
+        self.scope_node = node
+        self.visitchildren(node)
+        self.scope_type, self.scope_node = prev
+        return node
     
     def visit_ClassDefNode(self, node):
-        prev = self.scope_type
-        self.scope_type = 'class'
-        self.classnode = node
-        self.visitchildren(node)
-        self.scope_type = prev
-        del self.classnode
-        return node
+        return self.visit_scope(node, 'class')
 
     def visit_FuncDefNode(self, node):
-        prev = self.scope_type
-        self.scope_type = 'function'
-        self.visitchildren(node)
-        self.scope_type = prev
-        return node
+        return self.visit_scope(node, 'function')
+
+    def visit_CStructOrUnionDefNode(self, node):
+        return self.visit_scope(node, 'struct')
 
     # cdef variables
     def handle_bufferdefaults(self, decl):
         if not isinstance(decl.default, DictNode):
             raise PostParseError(decl.pos, ERR_BUF_DEFAULTS)
-        self.classnode.buffer_defaults_node = decl.default
-        self.classnode.buffer_defaults_pos = decl.pos
+        self.scope_node.buffer_defaults_node = decl.default
+        self.scope_node.buffer_defaults_pos = decl.pos
 
     def visit_CVarDefNode(self, node):
         # This assumes only plain names and pointers are assignable on
@@ -171,8 +174,8 @@ class PostParse(CythonTransform):
                     declbase = declbase.base
                 if isinstance(declbase, CNameDeclaratorNode):
                     if declbase.default is not None:
-                        if self.scope_type == 'class':
-                            if isinstance(self.classnode, CClassDefNode):
+                        if self.scope_type in ('class', 'struct'):
+                            if isinstance(self.scope_node, CClassDefNode):
                                 handler = self.specialattribute_handlers.get(decl.name)
                                 if handler:
                                     if decl is not declbase:
@@ -180,9 +183,10 @@ class PostParse(CythonTransform):
                                     handler(decl)
                                     continue # Remove declaration
                             raise PostParseError(decl.pos, ERR_CDEF_INCLASS)
+                        first_assignment = self.scope_type != 'module'
                         stats.append(SingleAssignmentNode(node.pos,
                             lhs=NameNode(node.pos, name=declbase.name),
-                            rhs=declbase.default, first=True))
+                            rhs=declbase.default, first=first_assignment))
                         declbase.default = None
                 newdecls.append(decl)
             node.declarators = newdecls
@@ -238,7 +242,7 @@ class PxdPostParse(CythonTransform):
         else:
             return node
 
-class ResolveOptions(CythonTransform):
+class InterpretCompilerDirectives(CythonTransform):
     """
     After parsing, options can be stored in a number of places:
     - #cython-comments at the top of the file (stored in ModuleNode)
@@ -246,29 +250,41 @@ class ResolveOptions(CythonTransform):
     - @cython.optionname decorators
     - with cython.optionname: statements
 
-    This transform is responsible for annotating each node with an
-    "options" attribute linking it to a dict containing the exact
-    options that are in effect for that node. Any corresponding decorators
-    or with statements are removed in the process.
+    This transform is responsible for interpreting these various sources
+    and store the option in two ways:
+    - Set the directives attribute of the ModuleNode for global directives.
+    - Use a CompilerDirectivesNode to override directives for a subtree.
+
+    (The first one is primarily to not have to modify with the tree
+    structure, so that ModuleNode stay on top.)
+
+    The directives are stored in dictionaries from name to value in effect.
+    Each such dictionary is always filled in for all possible directives,
+    using default values where no value is given by the user.
+
+    The available directives are controlled in Options.py.
 
     Note that we have to run this prior to analysis, and so some minor
     duplication of functionality has to occur: We manually track cimports
-    to correctly intercept @cython... and with cython...
+    and which names the "cython" module may have been imported to.
     """
+    special_methods = set(['declare', 'union', 'struct', 'typedef', 'sizeof', 'cast', 'address', 'pointer', 'compiled', 'NULL'])
 
     def __init__(self, context, compilation_option_overrides):
-        super(ResolveOptions, self).__init__(context)
+        super(InterpretCompilerDirectives, self).__init__(context)
         self.compilation_option_overrides = compilation_option_overrides
         self.cython_module_names = set()
         self.option_names = {}
 
+    # Set up processing and handle the cython: comments.
     def visit_ModuleNode(self, node):
         options = copy.copy(Options.option_defaults)
         options.update(node.option_comments)
         options.update(self.compilation_option_overrides)
         self.options = options
-        node.options = options
+        node.directives = options
         self.visitchildren(node)
+        node.cython_module_names = self.cython_module_names
         return node
 
     # Track cimports of the cython module.
@@ -285,19 +301,56 @@ class ResolveOptions(CythonTransform):
         if node.module_name == u"cython":
             newimp = []
             for pos, name, as_name, kind in node.imported_names:
-                if name in Options.option_types:
+                if (name in Options.option_types or 
+                        name in self.special_methods or
+                        PyrexTypes.parse_basic_type(name)):
+                    if as_name is None:
+                        as_name = name
                     self.option_names[as_name] = name
                     if kind is not None:
                         self.context.nonfatal_error(PostParseError(pos,
                             "Compiler option imports must be plain imports"))
-                    return None
                 else:
                     newimp.append((pos, name, as_name, kind))
-            node.imported_names = newimpo
+            if not newimp:
+                return None
+            node.imported_names = newimp
+        return node
+        
+    def visit_FromImportStatNode(self, node):
+        if node.module.module_name.value == u"cython":
+            newimp = []
+            for name, name_node in node.items:
+                if (name in Options.option_types or 
+                        name in self.special_methods or
+                        PyrexTypes.parse_basic_type(name)):
+                    self.option_names[name_node.name] = name
+                else:
+                    newimp.append((name, name_node))
+            if not newimp:
+                return None
+            node.items = newimp
         return node
 
+    def visit_SingleAssignmentNode(self, node):
+        if (isinstance(node.rhs, ImportNode) and
+                node.rhs.module_name.value == u'cython'):
+            node = CImportStatNode(node.pos, 
+                                   module_name = u'cython',
+                                   as_name = node.lhs.name)
+            self.visit_CImportStatNode(node)
+        else:
+            self.visitchildren(node)
+        return node
+            
+    def visit_NameNode(self, node):
+        if node.name in self.cython_module_names:
+            node.is_cython_module = True
+        else:
+            node.cython_attribute = self.option_names.get(node.name)
+        return node
+        
     def visit_Node(self, node):
-        node.options = self.options
         self.visitchildren(node)
         return node
 
@@ -306,37 +359,40 @@ class ResolveOptions(CythonTransform):
         # decorator), returns (optionname, value).
         # Otherwise, returns None
         optname = None
-        if isinstance(node, SimpleCallNode):
-            if (isinstance(node.function, AttributeNode) and
-                  isinstance(node.function.obj, NameNode) and
-                  node.function.obj.name in self.cython_module_names):
-                optname = node.function.attribute
-            elif (isinstance(node.function, NameNode) and
-                  node.function.name in self.option_names):
-                optname = self.option_names[node.function.name]
+        if isinstance(node, CallNode):
+            self.visit(node.function)
+            optname = node.function.as_cython_attribute()
 
         if optname:
             optiontype = Options.option_types.get(optname)
             if optiontype:
-                args = node.args
+                args, kwds = node.explicit_args_kwds()
                 if optiontype is bool:
-                    if len(args) != 1 or not isinstance(args[0], BoolNode):
+                    if kwds is not None or len(args) != 1 or not isinstance(args[0], BoolNode):
                         raise PostParseError(dec.function.pos,
                             'The %s option takes one compile-time boolean argument' % optname)
                     return (optname, args[0].value)
+                elif optiontype is dict:
+                    if len(args) != 0:
+                        raise PostParseError(dec.function.pos,
+                            'The %s option takes no prepositional arguments' % optname)
+                    return optname, dict([(key.value, value) for key, value in kwds.key_value_pairs])
                 else:
                     assert False
 
         return None
 
-    def visit_with_options(self, node, options):
+    def visit_with_options(self, body, options):
         oldoptions = self.options
         newoptions = copy.copy(oldoptions)
         newoptions.update(options)
         self.options = newoptions
-        node = self.visit_Node(node)
+        assert isinstance(body, StatListNode), body
+        retbody = self.visit_Node(body)
+        directive = CompilerDirectivesNode(pos=retbody.pos, body=retbody,
+                                           directives=newoptions)
         self.options = oldoptions
-        return node  
+        return directive
  
     # Handle decorators
     def visit_DefNode(self, node):
@@ -352,14 +408,15 @@ class ResolveOptions(CythonTransform):
                 else:
                     realdecs.append(dec)
             node.decorators = realdecs
-
+        
         if options:
             optdict = {}
             options.reverse() # Decorators coming first take precedence
             for option in options:
                 name, value = option
                 optdict[name] = value
-            return self.visit_with_options(node, optdict)
+            body = StatListNode(node.pos, stats=[node])
+            return self.visit_with_options(body, optdict)
         else:
             return self.visit_Node(node)
                                    
@@ -370,8 +427,7 @@ class ResolveOptions(CythonTransform):
             if node.target is not None:
                 raise PostParseError(node.pos, "Compiler option with statements cannot contain 'as'")
             name, value = option
-            self.visit_with_options(node.body, {name:value})
-            return node.body.stats
+            return self.visit_with_options(node.body, {name:value})
         else:
             return self.visit_Node(node)
 
@@ -395,7 +451,7 @@ class WithTransform(CythonTransform):
         finally:
             if EXC:
                 EXIT(None, None, None)
-    """, temps=[u'MGR', u'EXC', u"EXIT", u"SYS)"],
+    """, temps=[u'MGR', u'EXC', u"EXIT"],
     pipeline=[NormalizeTree(None)])
 
     template_with_target = TreeFragment(u"""
@@ -414,32 +470,32 @@ class WithTransform(CythonTransform):
         finally:
             if EXC:
                 EXIT(None, None, None)
-    """, temps=[u'MGR', u'EXC', u"EXIT", u"VALUE", u"SYS"],
+    """, temps=[u'MGR', u'EXC', u"EXIT", u"VALUE"],
     pipeline=[NormalizeTree(None)])
 
     def visit_WithStatNode(self, node):
-        excinfo_name = temp_name_handle('EXCINFO')
-        excinfo_namenode = NameNode(pos=node.pos, name=excinfo_name)
-        excinfo_target = NameNode(pos=node.pos, name=excinfo_name)
+        excinfo_temp = TempHandle(PyrexTypes.py_object_type)
         if node.target is not None:
             result = self.template_with_target.substitute({
                 u'EXPR' : node.manager,
                 u'BODY' : node.body,
                 u'TARGET' : node.target,
-                u'EXCINFO' : excinfo_namenode
-                }, pos = node.pos)
+                u'EXCINFO' : excinfo_temp.ref(node.pos)
+                }, pos=node.pos)
             # Set except excinfo target to EXCINFO
-            result.stats[4].body.stats[0].except_clauses[0].excinfo_target = excinfo_target
+            result.body.stats[4].body.stats[0].except_clauses[0].excinfo_target = (
+                excinfo_temp.ref(node.pos))
         else:
             result = self.template_without_target.substitute({
                 u'EXPR' : node.manager,
                 u'BODY' : node.body,
-                u'EXCINFO' : excinfo_namenode
-                }, pos = node.pos)
+                u'EXCINFO' : excinfo_temp.ref(node.pos)
+                }, pos=node.pos)
             # Set except excinfo target to EXCINFO
-            result.stats[4].body.stats[0].except_clauses[0].excinfo_target = excinfo_target
-        
-        return result.stats
+            result.body.stats[4].body.stats[0].except_clauses[0].excinfo_target = (
+                excinfo_temp.ref(node.pos))
+
+        return TempsBlockNode(node.pos, temps=[excinfo_temp], body=result)
 
 class DecoratorTransform(CythonTransform):
 
@@ -479,17 +535,24 @@ property NAME:
         node.analyse_declarations(self.env_stack[-1])
         self.visitchildren(node)
         return node
-
+        
     def visit_FuncDefNode(self, node):
         lenv = node.create_local_scope(self.env_stack[-1])
         node.body.analyse_control_flow(lenv) # this will be totally refactored
         node.declare_arguments(lenv)
+        for var, type_node in node.directive_locals.items():
+            if not lenv.lookup_here(var):   # don't redeclare args
+                type = type_node.analyse_as_type(lenv)
+                if type:
+                    lenv.declare_var(var, type, type_node.pos)
+                else:
+                    error(type_node.pos, "Not a type")
         node.body.analyse_declarations(lenv)
         self.env_stack.append(lenv)
         self.visitchildren(node)
         self.env_stack.pop()
         return node
-        
+    
     # Some nodes are no longer needed after declaration
     # analysis and can be dropped. The analysis was performed
     # on these nodes in a seperate recursive process from the
@@ -519,6 +582,7 @@ property NAME:
         return property
 
 class AnalyseExpressionsTransform(CythonTransform):
+
     def visit_ModuleNode(self, node):
         node.body.analyse_expressions(node.scope)
         self.visitchildren(node)
@@ -528,6 +592,54 @@ class AnalyseExpressionsTransform(CythonTransform):
         node.body.analyse_expressions(node.local_scope)
         self.visitchildren(node)
         return node
+        
+class AlignFunctionDefinitions(CythonTransform):
+    """
+    This class takes the signatures from a .pxd file and applies them to 
+    the def methods in a .py file. 
+    """
+    
+    def visit_ModuleNode(self, node):
+        self.scope = node.scope
+        self.visitchildren(node)
+        return node
+    
+    def visit_PyClassDefNode(self, node):
+        pxd_def = self.scope.lookup(node.name)
+        if pxd_def:
+            if pxd_def.is_cclass:
+                return self.visit_CClassDefNode(node.as_cclass(), pxd_def)
+            else:
+                error(node.pos, "'%s' redeclared" % node.name)
+                error(pxd_def.pos, "previous declaration here")
+                return None
+        self.visitchildren(node)
+        return node
+        
+    def visit_CClassDefNode(self, node, pxd_def=None):
+        if pxd_def is None:
+            pxd_def = self.scope.lookup(node.class_name)
+        if pxd_def:
+            outer_scope = self.scope
+            self.scope = pxd_def.type.scope
+        self.visitchildren(node)
+        if pxd_def:
+            self.scope = outer_scope
+        return node
+        
+    def visit_DefNode(self, node):
+        pxd_def = self.scope.lookup(node.name)
+        if pxd_def:
+            if pxd_def.is_cfunction:
+                node = node.as_cfunction(pxd_def)
+            else:
+                error(node.pos, "'%s' redeclared" % node.name)
+                error(pxd_def.pos, "previous declaration here")
+                return None
+        # Enable this when internal def functions are allowed. 
+        # self.visitchildren(node)
+        return node
+        
 
 class MarkClosureVisitor(CythonTransform):
     
@@ -576,3 +688,87 @@ class CreateClosureClasses(CythonTransform):
         return node
         
 
+class EnvTransform(CythonTransform):
+    """
+    This transformation keeps a stack of the environments. 
+    """
+    def __call__(self, root):
+        self.env_stack = [root.scope]
+        return super(EnvTransform, self).__call__(root)        
+    
+    def visit_FuncDefNode(self, node):
+        self.env_stack.append(node.local_scope)
+        self.visitchildren(node)
+        self.env_stack.pop()
+        return node
+
+
+class TransformBuiltinMethods(EnvTransform):
+
+    def visit_SingleAssignmentNode(self, node):
+        if node.declaration_only:
+            return None
+        else:
+            self.visitchildren(node)
+            return node
+    
+    def visit_AttributeNode(self, node):
+        return self.visit_cython_attribute(node)
+
+    def visit_NameNode(self, node):
+        return self.visit_cython_attribute(node)
+        
+    def visit_cython_attribute(self, node):
+        attribute = node.as_cython_attribute()
+        if attribute:
+            if attribute == u'compiled':
+                node = BoolNode(node.pos, value=True)
+            elif attribute == u'NULL':
+                node = NullNode(node.pos)
+            elif not PyrexTypes.parse_basic_type(attribute):
+                error(node.pos, u"'%s' not a valid cython attribute or is being used incorrectly" % attribute)
+        return node
+
+    def visit_SimpleCallNode(self, node):
+
+        # locals builtin
+        if isinstance(node.function, ExprNodes.NameNode):
+            if node.function.name == 'locals':
+                pos = node.pos
+                lenv = self.env_stack[-1]
+                items = [ExprNodes.DictItemNode(pos, 
+                                                key=ExprNodes.IdentifierStringNode(pos, value=var),
+                                                value=ExprNodes.NameNode(pos, name=var)) for var in lenv.entries]
+                return ExprNodes.DictNode(pos, key_value_pairs=items)
+
+        # cython.foo
+        function = node.function.as_cython_attribute()
+        if function:
+            if function == u'cast':
+                if len(node.args) != 2:
+                    error(node.function.pos, u"cast takes exactly two arguments")
+                else:
+                    type = node.args[0].analyse_as_type(self.env_stack[-1])
+                    if type:
+                        node = TypecastNode(node.function.pos, type=type, operand=node.args[1])
+                    else:
+                        error(node.args[0].pos, "Not a type")
+            elif function == u'sizeof':
+                if len(node.args) != 1:
+                    error(node.function.pos, u"sizeof takes exactly one argument" % function)
+                else:
+                    type = node.args[0].analyse_as_type(self.env_stack[-1])
+                    if type:
+                        node = SizeofTypeNode(node.function.pos, arg_type=type)
+                    else:
+                        node = SizeofVarNode(node.function.pos, operand=node.args[0])
+            elif function == 'address':
+                if len(node.args) != 1:
+                    error(node.function.pos, u"sizeof takes exactly one argument" % function)
+                else:
+                    node = AmpersandNode(node.function.pos, operand=node.args[0])
+            else:
+                error(node.function.pos, u"'%s' not a valid cython language construct" % function)
+        
+        self.visitchildren(node)
+        return node
